@@ -31,9 +31,10 @@ __date__ = "26/10/2021"
 import copy
 import glob
 import logging
-import os
-import threading
 import multiprocessing
+import os
+import tempfile
+import threading
 from functools import partial
 from multiprocessing import Pool
 from enum import IntEnum
@@ -496,6 +497,8 @@ class Dataset():
 
         _dir = self.dir if _dir is None else _dir
 
+        temp_dir = tempfile.mkdtemp()
+
         if not os.path.isdir(_dir):
             os.mkdir(_dir)
 
@@ -522,7 +525,7 @@ class Dataset():
 
         if self._in_memory:
             new_data = background_subtraction(self.running_data, bg_data, method).view(Data)
-            new_data.save(os.path.join(_dir, "data.hdf5"))
+            new_data.save(os.path.join(temp_dir, "data.hdf5"))
             urls = new_data.urls
         else:
             bg = numpy.zeros(self.running_data[0].shape, self.running_data.dtype)
@@ -578,9 +581,8 @@ class Dataset():
 
             if not self.operations_state[Operation.BS]:
                 return
-
             urls = self.running_data.apply_funcs([(background_subtraction_2D, [bg])],
-                                                 save=os.path.join(_dir, "data.hdf5"), text="Applying background subtraction",
+                                                 save=os.path.join(temp_dir, "data.hdf5"), text="Applying background subtraction",
                                                  operation=Operation.BS)
             if urls is None:
                 return
@@ -599,6 +601,8 @@ class Dataset():
             new_data = Data(urls.reshape(self.data.urls.shape), self.data.metadata,
                             self._in_memory)
 
+        new_data.save(os.path.join(_dir, "data.hdf5"))
+
         return Dataset(_dir=_dir, data=new_data, dims=self.__dims, transformation=self.transformation,
                        in_memory=self._in_memory)
 
@@ -621,13 +625,14 @@ class Dataset():
         self.running_data = self.get_data(indices)
 
         _dir = self.dir if _dir is None else _dir
+        temp_dir = tempfile.mkdtemp()
 
         if not os.path.isdir(_dir):
             os.mkdir(_dir)
 
         if self._in_memory:
             new_data = hot_pixel_removal_3D(self.running_data, kernel).view(Data)
-            new_data.save(os.path.join(_dir, "data.hdf5"))
+            new_data.save(os.path.join(temp_dir, "data.hdf5"))
             urls = new_data.urls
         else:
             urls = self.running_data.apply_funcs([(hot_pixel_removal_2D, [kernel])],
@@ -645,6 +650,8 @@ class Dataset():
             new_data = Data(urls.reshape(self.data.urls.shape), self.data.metadata,
                             self._in_memory)
 
+        new_data.save(os.path.join(_dir, "data.hdf5"))
+
         return Dataset(_dir=_dir, data=new_data, dims=self.__dims, transformation=self.transformation,
                        in_memory=self._in_memory)
 
@@ -667,13 +674,14 @@ class Dataset():
         self.running_data = self.get_data(indices)
 
         _dir = self.dir if _dir is None else _dir
+        temp_dir = tempfile.mkdtemp()
 
         if not os.path.isdir(_dir):
             os.mkdir(_dir)
 
         if self._in_memory:
             new_data = threshold_removal(self.running_data, bottom, top).view(Data)
-            new_data.save(os.path.join(_dir, "data.hdf5"))
+            new_data.save(os.path.join(temp_dir, "data.hdf5"))
             urls = new_data.urls
         else:
             urls = self.running_data.apply_funcs([(threshold_removal, [bottom, top])],
@@ -691,6 +699,8 @@ class Dataset():
             new_data = Data(urls.reshape(self.data.urls.shape), self.data.metadata,
                             self._in_memory)
 
+        new_data.save(os.path.join(_dir, "data.hdf5"))
+
         return Dataset(_dir=_dir, data=new_data, dims=self.__dims, transformation=self.transformation,
                        in_memory=self._in_memory)
 
@@ -1027,7 +1037,7 @@ class Dataset():
             H, vals, W = model.H, model.singular_values, model.W
         return vals if return_vals else (H, W)
 
-    def nica(self, num_components, chunksize=None, num_iter=500, indices=None):
+    def nica(self, num_components, chunksize=None, num_iter=500, error_step=None, indices=None):
         """
         Compute Non-negative Independent Component Analysis on the data.
         The method, first converts, if not already, the data into an hdf5 file object
@@ -1055,7 +1065,7 @@ class Dataset():
             chunksize = None
         model = NICA(self.data.convert_to_hdf5(bss_dir), num_components, chunksize, indices=indices)
         try:
-            model.fit_transform(max_iter=num_iter)
+            model.fit_transform(max_iter=num_iter, error_step=error_step)
             return numpy.abs(model.H), numpy.abs(model.W)
         finally:
             self.data._file.close()
@@ -1275,49 +1285,51 @@ class Dataset():
                     data, rindices = self.get_data(indices, [i, j], return_indices=True)
                     if len(data) < 3:
                         _logger.warning("Not enough values for fitting")
-                        break
-                    values = numpy.array(self.get_metadata_values(kind=dim.kind, key=dim.name, indices=indices, dimension=[i, j])).reshape(-1)
-                    if not data.in_memory:
-                        # Use chunks to load the data into memory
-                        start = [0, 0]
-                        chunks_1 = int(numpy.ceil(data.shape[1] / chunk_shape[0]))
-                        chunks_2 = int(numpy.ceil(data.shape[2] / chunk_shape[1]))
-                        img = numpy.empty((data.shape[1], data.shape[2]), dtype=data[0].dtype)
-                        io_utils.advancement_display(0, chunks_1 * chunks_2 * len(data), "Fitting rocking curves with " + dim.name + " " + str(dim.unique_values[j]))
-                        for chunk1 in range(chunks_1):
-                            for chunk2 in range(chunks_2):
-                                if not self.operations_state[Operation.FIT]:
-                                    return
-                                # Use multiprocessing to chunk the images
-                                cpus = multiprocessing.cpu_count()
-                                with Pool(cpus - 1) as p:
-                                    c_images = p.map(partial(chunk_image, start, chunk_shape), data)
-                                fitted_data = fit_data(numpy.asarray(c_images), values=values, int_thresh=int_thresh)
-                                for k in range(len(fitted_data)):
-                                    filename = os.path.join(_dir, "data_fit_" + self.dims.get(i).name + str(rindices[k]).zfill(4) + ".npy")
-                                    if (chunk1, chunk2) != (0, 0):
-                                        # If chunk is not the first, load image from disk
-                                        img = numpy.load(filename)
-                                    else:
-                                        urls.append(DataUrl(file_path=filename, scheme='fabio'))
-                                    img[start[0]:start[0] + chunk_shape[0],
-                                        start[1]:start[1] + chunk_shape[1]] = fitted_data[k]
-                                    numpy.save(filename, img)
-                                    io_utils.advancement_display(
-                                        chunk1 * chunks_2 * len(data) + chunk2 * len(data) + k + 1,
-                                        chunks_1 * chunks_2 * len(data),
-                                        "Fitting rocking curves with " + dim.name + " " + str(dim.unique_values[j]))
-                                start[1] = chunk_shape[1] * (chunk2 + 1)
-                            start[0] = chunk_shape[0] * (chunk1 + 1)
-                            start[1] = 0
+                        for url in data.urls:
+                            urls.append(url)
                     else:
-                        if not self.operations_state[Operation.FIT]:
-                            return
-                        fitted_data = fit_data(data, values=values, int_thresh=int_thresh, _tqdm=True)
-                        for k, image in enumerate(fitted_data):
-                            filename = os.path.join(_dir, "data_fit_" + self.dims.get(dimension[0][0]).name + str(rindices[k]).zfill(4) + ".npy")
-                            numpy.save(filename, image)
-                            urls.append(DataUrl(file_path=filename, scheme='fabio'))
+                        values = numpy.array(self.get_metadata_values(kind=dim.kind, key=dim.name, indices=indices, dimension=[i, j])).reshape(-1)
+                        if not data.in_memory:
+                            # Use chunks to load the data into memory
+                            start = [0, 0]
+                            chunks_1 = int(numpy.ceil(data.shape[1] / chunk_shape[0]))
+                            chunks_2 = int(numpy.ceil(data.shape[2] / chunk_shape[1]))
+                            img = numpy.empty((data.shape[1], data.shape[2]), dtype=data[0].dtype)
+                            io_utils.advancement_display(0, chunks_1 * chunks_2 * len(data), "Fitting rocking curves with " + dim.name + " " + str(dim.unique_values[j]))
+                            for chunk1 in range(chunks_1):
+                                for chunk2 in range(chunks_2):
+                                    if not self.operations_state[Operation.FIT]:
+                                        return
+                                    # Use multiprocessing to chunk the images
+                                    cpus = multiprocessing.cpu_count()
+                                    with Pool(cpus - 1) as p:
+                                        c_images = p.map(partial(chunk_image, start, chunk_shape), data)
+                                    fitted_data = fit_data(numpy.asarray(c_images), values=values, int_thresh=int_thresh)
+                                    for k in range(len(fitted_data)):
+                                        filename = os.path.join(_dir, "data_fit_" + self.dims.get(i).name + str(rindices[k]).zfill(4) + ".npy")
+                                        if (chunk1, chunk2) != (0, 0):
+                                            # If chunk is not the first, load image from disk
+                                            img = numpy.load(filename)
+                                        else:
+                                            urls.append(DataUrl(file_path=filename, scheme='fabio'))
+                                        img[start[0]:start[0] + chunk_shape[0],
+                                            start[1]:start[1] + chunk_shape[1]] = fitted_data[k]
+                                        numpy.save(filename, img)
+                                        io_utils.advancement_display(
+                                            chunk1 * chunks_2 * len(data) + chunk2 * len(data) + k + 1,
+                                            chunks_1 * chunks_2 * len(data),
+                                            "Fitting rocking curves with " + dim.name + " " + str(dim.unique_values[j]))
+                                    start[1] = chunk_shape[1] * (chunk2 + 1)
+                                start[0] = chunk_shape[0] * (chunk1 + 1)
+                                start[1] = 0
+                        else:
+                            if not self.operations_state[Operation.FIT]:
+                                return
+                            fitted_data = fit_data(data, values=values, int_thresh=int_thresh, _tqdm=True)
+                            for k, image in enumerate(fitted_data):
+                                filename = os.path.join(_dir, "data_fit_" + self.dims.get(dimension[0][0]).name + str(rindices[k]).zfill(4) + ".npy")
+                                numpy.save(filename, image)
+                                urls.append(DataUrl(file_path=filename, scheme='fabio'))
 
         self._lock.acquire()
         self.operations_state[Operation.FIT] = 0
@@ -1401,8 +1413,8 @@ class Data(numpy.ndarray):
                 # Create array as stack of images
                 img = utils.get_data(urls.flatten()[0])
                 data = numpy.empty((urls.flatten().shape[0], img.shape[0], img.shape[1]), img.dtype)
-                for i in range(len(urls.flatten())):
-                    data[i] = utils.get_data(urls.flatten()[i])
+                for i, url in enumerate(urls.flatten()):
+                    data[i] = utils.get_data(url)
             shape = list(urls.shape)
             shape.append(data.shape[-2])
             shape.append(data.shape[-1])
@@ -1539,7 +1551,6 @@ class Data(numpy.ndarray):
         _file = h5py.File(save, 'a')
         dataset_name = "dataset"
         new_shape = self.shape if new_shape is None else tuple(new_shape)
-
         if "dataset" in _file:
             if new_shape != _file["dataset"].shape:
                 _file.create_dataset("update_dataset", new_shape, dtype=self.dtype)
@@ -1549,7 +1560,6 @@ class Data(numpy.ndarray):
                     _file["update_dataset"][i] = img
             dataset_name = "update_dataset"
         else:
-            new_shape = self.shape if new_shape is None else new_shape
             _file.create_dataset("dataset", new_shape, dtype=self.dtype)
 
         for i in indices:
@@ -1596,16 +1606,17 @@ class Data(numpy.ndarray):
         :param indices: the indices of the values to save, defaults to None
         :type indices: Union[None,array_like], optional
         """
-        self.in_memory = True
+        if not hasattr(self, "in_memory") or self.in_memory is None:
+            self.in_memory = True
         urls = []
         if indices is None:
-            data = self
+            data = self.flatten() if self.urls is not None else self
         else:
-            data = self[indices]
+            data = self[indices].flatten() if self.urls is not None else self[indices]
 
         _file = h5py.File(path, 'a')
 
-        new_shape = self.shape if new_shape is None else tuple(new_shape)
+        new_shape = data.shape if new_shape is None else tuple(new_shape)
 
         if "dataset" not in _file:
             _file.create_dataset("dataset", self.shape, dtype=self.dtype)
@@ -1620,7 +1631,10 @@ class Data(numpy.ndarray):
         #     numpy.save(filename, img)
         #     urls.append(DataUrl(file_path=filename, scheme='fabio'))
         _file.close()
-        self.urls = numpy.asarray(urls)
+        if self.urls is not None:
+            self.urls = numpy.asarray(urls).reshape(self.urls.shape)
+        else:
+            self.urls = numpy.asarray(urls)
 
     def convert_to_hdf5(self, _dir):
         """
