@@ -116,8 +116,8 @@ class TestDataset(unittest.TestCase):
         """ Tests the background subtraction function with data in disk"""
 
         dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10, in_memory=False)
-        indices = [0, 1, 2, 3, 4]
-        bs_dataset = dataset.apply_background_subtraction(background=[5, 6, 7, 8, 9], indices=indices)
+        indices = [0, 2, 4, 5, 7]
+        bs_dataset = dataset.apply_background_subtraction(background=[1, 3, 6, 8, 9], indices=indices)
 
         self.assertEqual(bs_dataset.nframes, dataset.nframes)
         self.assertNotEqual(dataset.data.urls[0], bs_dataset.data.urls[0])
@@ -160,8 +160,8 @@ class TestDataset(unittest.TestCase):
         # Background data in memory
 
         dark_dataset = utils.createDataset(dark, in_memory=True)
-
         new_dataset = dataset.apply_background_subtraction(background=dark_dataset, method="mean")
+
         new_dataset.in_memory = True
         numpy.testing.assert_array_equal(expected, new_dataset.data)
 
@@ -293,6 +293,75 @@ class TestDataset(unittest.TestCase):
         self.assertEqual(new_dataset.nframes, 5)
         numpy.testing.assert_equal(dataset.get_data(indices)[0][:20, :20], new_dataset.data[0])
 
+    def test_roi_bs_in_memory(self):
+        """ Tests the background subtraction function with data in disk"""
+
+        roi_dataset = self.dataset.apply_roi(origin=[0, 0], size=[20, 20])
+        indices = [0, 1]
+        bs_dataset = roi_dataset.apply_background_subtraction(background=[2], indices=indices)
+
+        self.assertEqual(bs_dataset.nframes, self.dataset.nframes)
+        self.assertNotEqual(self.dataset.data.urls[0], bs_dataset.data.urls[0])
+        numpy.testing.assert_equal(bs_dataset.data[2], roi_dataset.data[2])
+
+    def test_roi_bs_in_disk(self):
+        """ Tests the background subtraction function with data in disk"""
+
+        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10, in_memory=False)
+        roi_dataset = dataset.apply_roi(origin=[0, 0], size=[20, 20])
+        indices = [0, 2, 4, 5, 7]
+        bs_dataset = roi_dataset.apply_background_subtraction(background=[1, 3, 6, 8, 9], indices=indices)
+        self.assertEqual(bs_dataset.nframes, dataset.nframes)
+        self.assertNotEqual(dataset.data.urls[0], bs_dataset.data.urls[0])
+        numpy.testing.assert_equal(bs_dataset.data[6], roi_dataset.data[6])
+
+    def test_roi_hp_in_memory(self):
+        """ Tests the background subtraction function with data in disk"""
+
+        roi_dataset = self.dataset.apply_roi(origin=[0, 0], size=[20, 20])
+        indices = [0, 1]
+        hp_dataset = roi_dataset.apply_hot_pixel_removal(indices=indices)
+
+        self.assertEqual(hp_dataset.nframes, self.dataset.nframes)
+        self.assertNotEqual(hp_dataset.data.urls[0], self.dataset.data.urls[0])
+        numpy.testing.assert_equal(self.dataset.data.metadata, hp_dataset.data.metadata)
+        numpy.testing.assert_equal(roi_dataset.data[2], hp_dataset.data[2])
+
+    def test_roi_hp_in_disk(self):
+        """ Tests the background subtraction function with data in disk"""
+
+        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10, in_memory=False)
+        roi_dataset = dataset.apply_roi(origin=[0, 0], size=[20, 20])
+        indices = [0, 2, 4, 5, 7]
+        bs_dataset = roi_dataset.apply_hot_pixel_removal(indices=indices)
+
+        self.assertEqual(bs_dataset.nframes, dataset.nframes)
+        self.assertNotEqual(dataset.data.urls[0], bs_dataset.data.urls[0])
+        numpy.testing.assert_equal(bs_dataset.data[6], roi_dataset.data[6])
+
+    def test_roi_threshold_in_memory(self):
+        """ Tests the threshold removal function with data in memory"""
+        roi_dataset = self.dataset.apply_roi(origin=[0, 0], size=[20, 20])
+        indices = [0, 1]
+        threshold_dataset = roi_dataset.apply_threshold_removal(indices=indices)
+
+        self.assertEqual(threshold_dataset.nframes, self.dataset.nframes)
+        self.assertNotEqual(threshold_dataset.data.urls[0], self.dataset.data.urls[0])
+        numpy.testing.assert_equal(self.dataset.data.metadata, threshold_dataset.data.metadata)
+        numpy.testing.assert_equal(roi_dataset.data[2], threshold_dataset.data[2])
+
+    def test_roi_threshold_in_disk(self):
+        """ Tests the threshold removal function with data in disk"""
+
+        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10, in_memory=False)
+        roi_dataset = dataset.apply_roi(origin=[0, 0], size=[20, 20])
+        indices = [0, 1, 2, 3, 4]
+        threshold_dataset = roi_dataset.apply_threshold_removal(indices=indices)
+
+        self.assertEqual(threshold_dataset.nframes, dataset.nframes)
+        self.assertNotEqual(threshold_dataset.data.urls[0], dataset.data.urls[0])
+        numpy.testing.assert_equal(roi_dataset.data[5], threshold_dataset.data[5])
+
     def test_find_shift(self):
         """ Tests the shift detection"""
         shift = self.dataset.find_shift()
