@@ -176,7 +176,7 @@ class BSSWidget(qt.QMainWindow):
             # Set actual values of W where threshold of filter is True
             W[self.indices] = self.W
             self.W = W
-        self._displayComponentsWidget.setComponents(self.comp, self.W, self.dataset.get_dimensions_values(), self.dataset.data.shape[:2])
+        self._displayComponentsWidget.setComponents(self.comp, self.W, self.dataset.get_dimensions_values(), self.dataset.data.shape[:-2])
 
     def _detectComp(self):
         self.detectButton.setEnabled(False)
