@@ -27,12 +27,12 @@ __authors__ = ["H. Payno", "J. Garriga"]
 __license__ = "MIT"
 __date__ = "26/04/2021"
 
-import numpy
 from functools import partial
 
 from silx.gui import qt
 from darfix.core.dataset import Dataset
 from darfix.core.dimension import Dimension, _METADATA_TYPES, _METADATA_TYPES_I, DEFAULT_METADATA
+import numpy
 import logging
 
 _logger = logging.getLogger(__file__)
@@ -43,7 +43,7 @@ class DimensionMapping(qt.QWidget):
     Widget used to define the number of dimension and with which values they are
     mapped
     """
-    _V_HEADERS = ['Axis', 'Kind', 'Name', 'Size', 'Tolerance', '', '']
+    _V_HEADERS = ['Axis', 'Kind', 'Name', 'Size', 'Range', 'Tolerance', '', '']
 
     fitSucceed = qt.Signal()
     """Signal emitted when the fit succeeds"""
@@ -58,6 +58,7 @@ class DimensionMapping(qt.QWidget):
         self._table.setColumnCount(len(self._V_HEADERS))
         self._table.setHorizontalHeaderLabels(self._V_HEADERS)
         header = self._table.horizontalHeader()
+        header.setMinimumSectionSize(50)
         if qt.qVersion() < "5.0":
             setResizeMode = header.setResizeMode
         else:
@@ -70,9 +71,9 @@ class DimensionMapping(qt.QWidget):
         self._table.verticalHeader().hide()
         self._dims = {}
 
-        self.layout().addWidget(self._table, 0, 0, 6, 6)
-        self._addButton = qt.QPushButton('add', parent=self)
-        self.layout().addWidget(self._addButton, 6, 5, 1, 1)
+        self.layout().addWidget(self._table, 0, 0, 6, 10)
+        self._addButton = qt.QPushButton('Add', parent=self)
+        self.layout().addWidget(self._addButton, 6, 6, 1, 1)
 
         # connect Signal/SLOT
         self._addButton.pressed.connect(self.addDim)
@@ -161,7 +162,7 @@ class DimensionWidget(DimensionMapping):
         self.indices = None
         self.bg_indices = None
         self.bg_dataset = None
-        self.metadata_type = _METADATA_TYPES['default']
+        self.metadata_type = _METADATA_TYPES['positioner']
         self.tolerance = 1e-9
 
         metadataTypeLabel = qt.QLabel("Metadata type: ")
@@ -208,7 +209,7 @@ class DimensionWidget(DimensionMapping):
         return self._dataset
 
     def getDataset(self):
-        return self._update_dataset, self.indices, self.bg_indices, self. bg_dataset
+        return self._update_dataset, self.indices, self.bg_indices, self.bg_dataset
 
     def _updateKind(self, metadata_type=None):
         if metadata_type is None:
@@ -230,7 +231,7 @@ class DimensionWidget(DimensionMapping):
             return
         self.dataset.clear_dims()
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
 
         :param dataset: the dataset for which we want to define the
@@ -238,6 +239,7 @@ class DimensionWidget(DimensionMapping):
         :type dataset: `Dataset`
         """
         assert isinstance(dataset, Dataset)
+        self._parent = parent
         self._dataset = dataset
         self._update_dataset = dataset
         self.indices = indices
@@ -247,6 +249,12 @@ class DimensionWidget(DimensionMapping):
             for widget in self._dims.values():
                 widget._setMetadata(self.dataset.get_data().metadata[0])
 
+    def _updateDataset(self, widget, dataset):
+        del self._dataset
+        self._dataset = dataset
+        self._update_dataset = dataset
+        self._parent._updateDataset(widget, dataset)
+
     def _find_dimensions(self):
         """
         Automatically find all dimensions that change through the dataset.
@@ -279,10 +287,30 @@ class DimensionWidget(DimensionMapping):
                 for dimension in self.dims.values():
                     axis = dimension.axis
                     dimension = dimension.dim
-                    values = numpy.unique([data.get_value(kind=dimension.kind,
-                                           name=dimension.name)[0] for data in self.dataset.get_data().metadata])
-                    dimension.set_unique_values(values)
+                    values = self.dataset.get_metadata_values(kind=dimension.kind, key=dimension.name)
+                    indices = numpy.unique(values, return_index=True)[1]
+                    dimension.set_unique_values([values[index] for index in sorted(indices)])
                     self.dataset.add_dim(axis=axis, dim=dimension)
+                try:
+                    self._update_dataset = self.dataset.reshape_data()
+                    ndim = self._update_dataset.dims.ndim
+                    for axis, dimension in self._update_dataset.dims:
+                        # Flip axis to be consistent with the data shape
+                        not_axis = ndim - axis - 1
+                        if self._update_dataset.dims.ndim > 1:
+                            metadata = numpy.swapaxes(self._update_dataset.data.metadata, ndim - 1, not_axis)
+                            for axis in range(ndim - 1):
+                                metadata = metadata[0]
+                        else:
+                            metadata = self._update_dataset.data.metadata
+                        values = [data.get_value(kind=dimension.kind,
+                                  name=dimension.name)[0] for data in metadata]
+                        dimension.set_unique_values(values)
+                except ValueError:
+                    for axis, dimension in self.dataset.dims:
+                        values = numpy.unique(self.dataset.get_metadata_values(kind=dimension.kind, key=dimension.name))
+                        dimension.set_unique_values(values)
+                    self._update_dataset = self.dataset.reshape_data()
             else:
                 raise ValueError("Cannot fit without dimensions")
             super().clear()
@@ -318,11 +346,10 @@ class DimensionWidget(DimensionMapping):
             assert type(axis) is int
             assert isinstance(dim, Dimension)
             self.addDim(axis=axis, dim=dim)
-            if self.dataset is not None and len(self.dataset.data.metadata) > 0:
+            if self.dataset is not None and len(self.dataset.get_data().metadata) > 0:
                 if not dim.unique_values:
-                    values = numpy.unique(
-                        [data.get_value(kind=dim.kind, name=dim.name)[0]
-                         for data in self.dataset.data.metadata])
+                    values = [data.get_value(kind=dim.kind, name=dim.name)[0]
+                              for data in self.dataset.get_data().metadata]
                     dim.set_unique_values(values)
             self.dataset.add_dim(axis=axis, dim=dim)
 
@@ -334,7 +361,7 @@ class DimensionWidget(DimensionMapping):
         """
         widget = super().addDim(axis, dim)
         if self.dataset is not None and len(self.dataset.data.metadata) > 0:
-            widget._setMetadata(self.dataset.data.metadata[0])
+            widget._setMetadata(self.dataset.get_data().metadata[0])
 
         return widget
 
@@ -377,7 +404,7 @@ class _DimensionItem(Dimension, qt.QWidget):
 
         @property
         def value(self):
-            return self._sizeSP.value
+            return self._sizeSP.value()
 
         def setValue(self, size, editable=True):
             """
@@ -410,6 +437,49 @@ class _DimensionItem(Dimension, qt.QWidget):
         def _valueHasChanged(self, value):
             self.valueChanged.emit(value)
 
+    class _RangeWidget(qt.QWidget):
+
+        def __init__(self, parent):
+            qt.QWidget.__init__(self, parent)
+            self.setLayout(qt.QHBoxLayout())
+            self._minLE = qt.QLineEdit(parent=self)
+            self._minLE.setPlaceholderText("Start")
+            self._maxLE = qt.QLineEdit(parent=self)
+            self._maxLE.setPlaceholderText("Stop")
+            self._stepLE = qt.QLineEdit(parent=self)
+            self._stepLE.setPlaceholderText("Step")
+            self._minLE.setValidator(qt.QDoubleValidator())
+            self._maxLE.setValidator(qt.QDoubleValidator())
+            validator = qt.QDoubleValidator()
+            validator.setBottom(0)
+            self._stepLE.setValidator(validator)
+            self.layout().addWidget(self._minLE)
+            self.layout().addWidget(self._maxLE)
+            self.layout().addWidget(self._stepLE)
+
+            self.layout().setContentsMargins(0, 0, 0, 0)
+
+        @property
+        def value(self):
+            if self._minLE.text() != "" and self._maxLE.text():
+                step = float(self._stepLE.text()) if self._stepLE.text() != "" else 0
+                return [float(self._minLE.text()), float(self._maxLE.text()), step]
+            else:
+                return None
+
+        def setValue(self, _range):
+            """
+
+            :param int or None _range
+            """
+            assert type(_range) is list
+            self._minLE.setText(str(numpy.round(_range[0], 5)))
+            self._maxLE.setText(str(numpy.round(_range[1], 5)))
+            self._stepLE.setText(str(numpy.round(_range[2], 5)))
+            self._minLE.setCursorPosition(0)
+            self._minLE.setCursorPosition(0)
+            self._minLE.setCursorPosition(0)
+
     def __init__(self, parent, table, row):
         """
 
@@ -427,16 +497,23 @@ class _DimensionItem(Dimension, qt.QWidget):
         self._axis.setMinimum(0)
         # kind
         self._kindCB = qt.QComboBox(parent=self)
+        self._kindCB.setMinimumWidth(100)
         for _kindName in _METADATA_TYPES:
             self._kindCB.addItem(_kindName)
         # name
         self._namesCB = qt.QComboBox(parent=self)
+        self._namesCB.setMinimumWidth(100)
         # size
         self._sizeWidget = self._SizeWidget(parent=self)
         self._sizeWidget.setMinimum(0)
         self._setSize(0)
+        # range
+        self._rangeWidget = self._RangeWidget(parent=self)
+        self._rangeWidget.setMinimumWidth(100)
+        self._rangeWidget.setMaximumWidth(200)
         # tolerance
         self._toleranceLE = qt.QLineEdit(parent=self)
+        self._toleranceLE.setMaximumWidth(80)
         validator = qt.QDoubleValidator()
         validator.setBottom(0)
         self._toleranceLE.setValidator(validator)
@@ -480,11 +557,14 @@ class _DimensionItem(Dimension, qt.QWidget):
 
     def showUniqueNames(self):
         title = "%s - %s" % (_METADATA_TYPES_I[self.kind], self.name)
-        ptp = float(self.unique_values[-1]) - float(self.unique_values[0])
-        unique_values_str = []
-        [unique_values_str.append(str(val)) for val in self.unique_values]
-        msg = 'Unique values: [' + ', '.join(unique_values_str) + ']\n Ptp: ' + str(ptp)
-        qt.QMessageBox.information(self, title, msg)
+        if len(self.unique_values):
+            ptp = float(self.unique_values[-1]) - float(self.unique_values[0])
+            unique_values_str = []
+            [unique_values_str.append(str(val)) for val in self.unique_values]
+            msg = 'Unique values: [' + ', '.join(unique_values_str) + ']\n Ptp: ' + str(ptp)
+            qt.QMessageBox.information(self, title, msg)
+        else:
+            qt.QMessageBox.information(self, title, "No values were found, try fitting")
 
     def setUniqueValues(self, values):
         super().set_unique_values(values)
@@ -506,6 +586,8 @@ class _DimensionItem(Dimension, qt.QWidget):
             self._namesCB.addItem(dim.name)
         if dim.size is not None:
             self._sizeWidget.setValue(dim.size)
+        if dim.range is not None:
+            self._rangeWidget.setValue(dim.range)
         self._setTolerance(str(dim.tolerance))
         self.setUniqueValues(dim.unique_values)
 
@@ -529,11 +611,15 @@ class _DimensionItem(Dimension, qt.QWidget):
 
     @property
     def dim(self):
-        return Dimension(name=self.name, kind=self.kind, size=self.size, tolerance=self.tolerance)
+        return Dimension(name=self.name, kind=self.kind, size=self.size, _range=self.range, tolerance=self.tolerance)
 
     @property
     def size(self):
-        return self._sizeWidget.value()
+        return self._sizeWidget.value
+
+    @property
+    def range(self):
+        return self._rangeWidget.value
 
     @property
     def name(self):
@@ -547,6 +633,10 @@ class _DimensionItem(Dimension, qt.QWidget):
         Dimension.set_size(self, size)
         self._sizeWidget.setValue(size=size)
 
+    def _setRange(self, _range):
+        Dimension.set_range(self, _range)
+        self._rangeWidget.setValue(_range)
+
     def setNames(self, names):
         self._namesCB.clear()
         for name in sorted(list(names)):
@@ -558,7 +648,7 @@ class _DimensionItem(Dimension, qt.QWidget):
     def embedInTable(self, table, row):
         self.__row = row
         for column, widget in enumerate((self._axis, self._kindCB, self._namesCB, self._sizeWidget,
-                                         self._toleranceLE, self._infoButton, self._rmButton)):
+                                         self._rangeWidget, self._toleranceLE, self._infoButton, self._rmButton)):
             table.setCellWidget(row, column, widget)
 
     def _updateNames(self, *args, **kwargs):
