@@ -93,6 +93,7 @@ class GrainPlotWidget(qt.QMainWindow):
         levelsLayout.addWidget(self._computeContoursB, 1, 2, 1, 1)
         levelsLayout.addWidget(self._contoursPlot, 2, 0, 1, 3)
         self._levelsWidget.setLayout(levelsLayout)
+        self._levelsWidget.hide()
         self._mosaicityPlot = Plot2D(parent=self)
         self._exportButton = qt.QPushButton("Export maps")
         self._exportButton.setEnabled(False)
@@ -109,32 +110,33 @@ class GrainPlotWidget(qt.QMainWindow):
         widget.setSizePolicy(qt.QSizePolicy.Minimum, qt.QSizePolicy.Minimum)
         self.setCentralWidget(widget)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
         Dataset setter.
 
         :param Dataset dataset: dataset
         """
-        self.dataset = dataset
+        self._parent = parent
+        self._dataset = dataset
         self.indices = indices
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
         for i in range(len(Method)):
             self._methodCB.model().item(i).setEnabled(False)
         scale = 100
-        if self.dataset.transformation:
-            transformation = self.dataset.transformation.transformation
+        if self._dataset.transformation:
+            transformation = self._dataset.transformation.transformation
             px = transformation[0][0][0]
             py = transformation[1][0][0]
-            xscale = (transformation[0][-1][-1] - px) / len(transformation[0][0])
-            yscale = (transformation[1][-1][-1] - py) / len(transformation[1][0])
+            xscale = (transformation[0][-1][-1] - px) / transformation[0].shape[1]
+            yscale = (transformation[1][-1][-1] - py) / transformation[1].shape[0]
             self.origin = (px, py)
             self.scale = (xscale, yscale)
-        if self.dataset.dims.ndim == 2:
+        if self._dataset.dims.ndim == 2:
             self._curves = {}
-            self.ori_dist, self.hsv_key = self.dataset.compute_mosaicity_colorkey()
-            xdim = self.dataset.dims.get(1)
-            ydim = self.dataset.dims.get(0)
+            self.ori_dist, self.hsv_key = self._dataset.compute_mosaicity_colorkey()
+            xdim = self._dataset.dims.get(1)
+            ydim = self._dataset.dims.get(0)
             xscale = (xdim.unique_values[-1] - xdim.unique_values[0]) / (xdim.size - 1)
             yscale = (ydim.unique_values[-1] - ydim.unique_values[0]) / (ydim.size - 1)
 
@@ -147,7 +149,7 @@ class GrainPlotWidget(qt.QMainWindow):
             self._computeContoursB.clicked.connect(self._computeContours)
             self._methodCB.model().item(4).setEnabled(True)
             self._methodCB.setCurrentIndex(4)
-        self._thread = OperationThread(self, self.dataset.apply_moments)
+        self._thread = OperationThread(self, self._dataset.apply_moments)
         self._thread.setArgs(self.indices)
         self._thread.finished.connect(self._updateData)
         self._thread.start()
@@ -155,7 +157,7 @@ class GrainPlotWidget(qt.QMainWindow):
             self._plotWidget.layout().itemAt(i).widget().setParent(None)
 
         self._plots = []
-        for axis, dim in self.dataset.dims:
+        for axis, dim in self._dataset.dims:
             self._plots += [Plot2D(parent=self)]
             self._plots[-1].setGraphTitle(dim.name)
             self._plots[-1].setDefaultColormap(Colormap(name='viridis'))
@@ -169,7 +171,7 @@ class GrainPlotWidget(qt.QMainWindow):
         if self._thread.data is not None:
             self._moments = self._thread.data
             self._updatePlot(self._methodCB.currentText())
-            rg = len(Method) if self.dataset.dims.ndim > 1 else 4
+            rg = len(Method) if self._dataset.dims.ndim > 1 else 4
             for i in range(rg):
                 self._methodCB.model().item(i).setEnabled(True)
             self._methodCB.setCurrentIndex(0)
@@ -177,13 +179,17 @@ class GrainPlotWidget(qt.QMainWindow):
         else:
             print("\nComputation aborted")
 
+    def _updateDataset(self, widget, dataset):
+        self._parent._updateDataset(widget, dataset)
+        self._dataset = dataset
+
     def _checkboxStateChanged(self, state):
         """
         Update widgets linked to the checkbox state
         """
         scale = 100
-        xdim = self.dataset.dims.get(1)
-        ydim = self.dataset.dims.get(0)
+        xdim = self._dataset.dims.get(1)
+        ydim = self._dataset.dims.get(0)
         xsize = xdim.size - 1
         ysize = ydim.size - 1
         xscale = (xdim.unique_values[-1] - xdim.unique_values[0]) / xsize
@@ -205,8 +211,8 @@ class GrainPlotWidget(qt.QMainWindow):
                 levels.append(i)
 
             colors = self._curvesColormap.applyToData(levels)
-            xdim = self.dataset.dims.get(1)
-            ydim = self.dataset.dims.get(0)
+            xdim = self._dataset.dims.get(1)
+            ydim = self._dataset.dims.get(0)
             self._curves = {}
             for ipolygon, polygon in enumerate(polygons):
                 # iso contours
@@ -251,11 +257,11 @@ class GrainPlotWidget(qt.QMainWindow):
             self._plotWidget.hide()
         elif method == Method.FWHM:
             self._plotWidget.show()
-            if self.dataset.transformation is not None:
-                label = self.dataset.transformation.label
+            if self._dataset.transformation is not None:
+                label = self._dataset.transformation.label
                 for i, plot in enumerate(self._plots):
                     plot.addImage(darfix.config.FWHM_VAL * (numpy.rot90(self._moments[i][1], 3)
-                                  if self.dataset.transformation.rotate else self._moments[i][0]),
+                                  if self._dataset.transformation.rotate else self._moments[i][1]),
                                   origin=self.origin, scale=self.scale,
                                   xlabel=label, ylabel=label)
             else:
@@ -263,10 +269,10 @@ class GrainPlotWidget(qt.QMainWindow):
                     plot.addImage(darfix.config.FWHM_VAL * self._moments[i][1])
         elif method == Method.COM:
             self._plotWidget.show()
-            if self.dataset.transformation is not None:
-                label = self.dataset.transformation.label
+            if self._dataset.transformation is not None:
+                label = self._dataset.transformation.label
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(numpy.rot90(self._moments[i][0], 3) if self.dataset.transformation.rotate
+                    plot.addImage(numpy.rot90(self._moments[i][0], 3) if self._dataset.transformation.rotate
                                   else self._moments[i][0], origin=self.origin, scale=self.scale,
                                   xlabel=label, ylabel=label)
             else:
@@ -274,10 +280,10 @@ class GrainPlotWidget(qt.QMainWindow):
                     plot.addImage(self._moments[i][0], xlabel='pixels', ylabel='pixels')
         elif method == Method.SKEWNESS:
             self._plotWidget.show()
-            if self.dataset.transformation is not None:
-                label = self.dataset.transformation.label
+            if self._dataset.transformation is not None:
+                label = self._dataset.transformation.label
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(numpy.rot90(self._moments[i][2], 3) if self.dataset.transformation.rotate
+                    plot.addImage(numpy.rot90(self._moments[i][2], 3) if self._dataset.transformation.rotate
                                   else self._moments[i][2], origin=self.origin, scale=self.scale,
                                   xlabel=label, ylabel=label)
             else:
@@ -285,22 +291,22 @@ class GrainPlotWidget(qt.QMainWindow):
                     plot.addImage(self._moments[i][2], xlabel='pixels', ylabel='pixels')
         elif method == Method.KURTOSIS:
             self._plotWidget.show()
-            if self.dataset.transformation is not None:
-                label = self.dataset.transformation.label
+            if self._dataset.transformation is not None:
+                label = self._dataset.transformation.label
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(numpy.rot90(self._moments[i][3], 3) if self.dataset.transformation.rotate
+                    plot.addImage(numpy.rot90(self._moments[i][3], 3) if self._dataset.transformation.rotate
                                   else self._moments[i][3], origin=self.origin, scale=self.scale,
                                   xlabel=label, ylabel=label)
             else:
                 for i, plot in enumerate(self._plots):
-                    plot.addImage(self._moments[i][0], xlabel='pixels', ylabel='pixels')
+                    plot.addImage(self._moments[i][3], xlabel='pixels', ylabel='pixels')
         elif method == Method.MOSAICITY:
             try:
                 self._plotWidget.hide()
-                if self.dataset.transformation:
-                    label = self.dataset.transformation.label
+                if self._dataset.transformation:
+                    label = self._dataset.transformation.label
                     image = hsv_to_rgb(self._computeMosaicity())
-                    self._mosaicityPlot.addImage(numpy.rot90(image, 3) if self.dataset.transformation.rotate
+                    self._mosaicityPlot.addImage(numpy.rot90(image, 3) if self._dataset.transformation.rotate
                                                  else image, origin=self.origin, scale=self.scale,
                                                  xlabel=label, ylabel=label)
                 else:
@@ -325,7 +331,7 @@ class GrainPlotWidget(qt.QMainWindow):
         """
         Creates dictionay with maps information and exports it to a nexus file
         """
-        if self.dataset and self.dataset.dims.ndim > 1:
+        if self._dataset and self._dataset.dims.ndim > 1:
             nx = {
                 "entry": {
                     "MOSAICITY": {
@@ -355,15 +361,13 @@ class GrainPlotWidget(qt.QMainWindow):
                 "@default": "entry"
             }
 
-            for axis, dim in self.dataset.dims:
-                nx["entry"]["maps"].update(
-                    {
-                        Method.COM.name: self._moments[axis][0],
-                        Method.FWHM.name: self._moments[axis][1],
-                        Method.SKEWNESS.name: self._moments[axis][2],
-                        Method.KURTOSIS.name: self._moments[axis][3]
-                    }
-                )
+            for axis, dim in self._dataset.dims:
+                nx["entry"]["maps"][dim.name] = {
+                    Method.COM.name: self._moments[axis][0],
+                    Method.FWHM.name: self._moments[axis][1],
+                    Method.SKEWNESS.name: self._moments[axis][2],
+                    Method.KURTOSIS.name: self._moments[axis][3]
+                }
         else:
             nx = {
                 "entry": {
