@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "23/09/2021"
+__date__ = "28/10/2021"
 
 
 from silx.gui import qt
@@ -58,9 +58,10 @@ class MagnificationWidget(qt.QMainWindow):
         self._checkbox10x = qt.QCheckBox("10x magnification")
         self._checkboxManual = qt.QCheckBox("Manual magnification:")
         self._topographyCheckbox = qt.QCheckBox("Topography (obpitch)")
-        self._topographyCheckbox.hide()
+        self._centerAxesCheckbox = qt.QCheckBox("Center axes")
+        self._centerAxesCheckbox.setChecked(True)
         self._orientationCB = qt.QComboBox()
-        self._orientationCB.addItems(["Horizontal", "Vertical"])
+        self._orientationCB.addItems(["Vertical", "Horizontal"])
         topographyAxis = qt.QLabel("Topography axis: ")
         self._manualLE = qt.QLineEdit(parent=self)
         self._manualLE.setEnabled(False)
@@ -84,6 +85,7 @@ class MagnificationWidget(qt.QMainWindow):
         self._topographyWidget.setMaximumHeight(40)
         layout.addWidget(self._topographyWidget, alignment=qt.Qt.AlignRight)
         layout.addWidget(self._okButton)
+        layout.addWidget(self._centerAxesCheckbox)
 
         # self._okButton.pressed.connect(self._saveMagnification)
         self._checkbox2x.stateChanged.connect(self._check2x)
@@ -96,12 +98,13 @@ class MagnificationWidget(qt.QMainWindow):
 
         self.setCentralWidget(widget)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
         Dataset setter.
 
         :param Dataset dataset: dataset
         """
+        self.parent = parent
         self.dataset = dataset
         self.indices = indices
         self.bg_indices = bg_indices
@@ -115,9 +118,6 @@ class MagnificationWidget(qt.QMainWindow):
             msg.exec_()
         else:
             self._okButton.setEnabled(True)
-            for axis, dim in self.dataset.dims:
-                if dim.name == "obpitch":
-                    self._topographyCheckbox.show()
 
     @property
     def magnification(self):
@@ -140,6 +140,10 @@ class MagnificationWidget(qt.QMainWindow):
     def getDataset(self):
         return self.dataset, self.indices, self.bg_indices, self.bg_dataset
 
+    def _updateDataset(self, widget, dataset):
+        self.parent._updateDataset(widget, dataset)
+        self.dataset = dataset
+
     def _checkManual(self, checked):
         if checked:
             self._checkbox2x.setChecked(False)
@@ -182,7 +186,9 @@ class MagnificationWidget(qt.QMainWindow):
                 return
         self._magnification = float(magnification)
 
-        self.dataset.compute_transformation(self.magnification, topography=[self._topographyCheckbox.isChecked(),
-                                            self._orientationCB.currentIndex()])
+        self.dataset.compute_transformation(
+            self.magnification, topography=[
+                self._topographyCheckbox.isChecked(), self._orientationCB.currentIndex()],
+            center=self._centerAxesCheckbox.isChecked())
 
         self.sigComputed.emit()
