@@ -37,11 +37,12 @@ class OperationThread(qt.QThread):
     Given a function and a set of arguments, it calls it whenever the thread
     is started.
     """
-    def __init__(self, function):
-        qt.QThread.__init__(self)
+    def __init__(self, parent, function):
+        qt.QThread.__init__(self, parent=parent)
         self.func = function
         self.args = []
         self.kwargs = {}
+        self.data = None
 
     def setArgs(self, *args, **kwargs):
         """
