@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "29/06/2021"
+__date__ = "07/12/2021"
 
 import numpy
 import logging
@@ -36,6 +36,7 @@ from silx.gui.colors import Colormap
 from silx.gui.plot import StackView, Plot1D, Plot2D
 from .utils import ChooseDimensionDock
 from silx.utils.enum import Enum as _Enum
+from silx.io.dictdump import dicttonx
 
 import darfix
 from darfix.core.mapping import fit_rocking_curve, compute_peak_position
@@ -72,13 +73,6 @@ class RockingCurvesWidget(qt.QMainWindow):
         self._plot = Plot1D(parent=self)
         self._plot.setGraphTitle("Rocking curves")
         self._plot.setGraphXLabel("Degrees")
-        self._plotMaps = Plot2D(self)
-        self._plotMaps.setDefaultColormap(Colormap(name='cividis', normalization='linear'))
-        self._methodCB = qt.QComboBox(self)
-        self._methodCB.addItems(Method.values())
-        self._methodCB.currentTextChanged.connect(self._updatePlot)
-        self._plotMaps.hide()
-        self._methodCB.hide()
         self._chooseDimensionDock = ChooseDimensionDock(self)
         intLabel = qt.QLabel("Intensity threshold:")
         self._intThresh = "15"
@@ -98,6 +92,16 @@ class RockingCurvesWidget(qt.QMainWindow):
         self._chooseDimensionDock.hide()
         self._chooseDimensionDock.widget.filterChanged.connect(self._filterStack)
         self._chooseDimensionDock.widget.stateDisabled.connect(self._wholeStack)
+        self._plotMaps = Plot2D(self)
+        self._plotMaps.setDefaultColormap(Colormap(name='cividis', normalization='linear'))
+        self._plotMaps.hide()
+        self._methodCB = qt.QComboBox(self)
+        self._methodCB.addItems(Method.values())
+        self._methodCB.currentTextChanged.connect(self._updatePlot)
+        self._methodCB.hide()
+        self._exportButton = qt.QPushButton("Export maps")
+        self._exportButton.hide()
+        self._exportButton.clicked.connect(self.exportMaps)
 
         layout.addWidget(self._sv, 0, 0, 1, 2)
         layout.addWidget(self._plot, 0, 2, 1, 2)
@@ -108,6 +112,7 @@ class RockingCurvesWidget(qt.QMainWindow):
         layout.addWidget(self._abortFit, 2, 2, 1, 2)
         layout.addWidget(self._methodCB, 3, 0, 1, 4)
         layout.addWidget(self._plotMaps, 4, 0, 1, 4)
+        layout.addWidget(self._exportButton, 5, 0, 1, 5)
         self._abortFit.hide()
         self.addDockWidget(qt.Qt.RightDockWidgetArea, self._chooseDimensionDock)
         widget.setLayout(layout)
@@ -298,6 +303,7 @@ class RockingCurvesWidget(qt.QMainWindow):
             self._updatePlot(self._methodCB.currentText())
             self._plotMaps.show()
             self._methodCB.show()
+            self._exportButton.show()
         else:
             print("\nCorrection aborted")
 
@@ -351,3 +357,50 @@ class RockingCurvesWidget(qt.QMainWindow):
     @property
     def dimension(self):
         return self._dimension
+
+    def exportMaps(self):
+        """
+        Creates dictionay with maps information and exports it to a nexus file
+        """
+
+        data = self._update_dataset.get_data(self.indices)
+        entry = "entry"
+
+        if self._dimension is not None:
+            dim = self.dataset.dims.get(self._dimension[0][0])
+            entry += "_" + dim.name
+
+        std = numpy.std(data, axis=0)
+        std[numpy.isnan(std)] = 0
+        nx = {
+            entry: {
+                "data": {
+                    ">" + Method.INTENSITY.name: "../maps/" + Method.INTENSITY.name,
+                    "@signal": Method.INTENSITY.name,
+                    "@NX_class": "NXdata"
+                },
+                "maps": {
+                    Method.INTENSITY.name: self._update_dataset.zsum(),
+                    Method.FWHM.name: darfix.config.FWHM_VAL * std,
+                    Method.PEAK.name: self.com,
+                    Method.RESIDUALS.name: numpy.sqrt(numpy.subtract(
+                        self._update_dataset.zsum(self.indices),
+                        self.dataset.zsum(self.indices)
+                    ) ** 2),
+                    "@NX_class": "NXcollection"
+                },
+                "@NX_class": "NXentry",
+                "@default": "data",
+            },
+            "@NX_class": "NXroot",
+            "@default": "entry"
+        }
+
+        fileDialog = qt.QFileDialog()
+
+        fileDialog.setFileMode(fileDialog.AnyFile)
+        fileDialog.setAcceptMode(fileDialog.AcceptSave)
+        fileDialog.setOption(fileDialog.DontUseNativeDialog)
+        fileDialog.setDefaultSuffix(".h5")
+        if fileDialog.exec_():
+            dicttonx(nx, fileDialog.selectedFiles()[0])
