@@ -113,20 +113,26 @@ class ROISelectionWidget(qt.QWidget):
         dock.visibilityChanged.connect(roiDockVisibilityChanged)
         plot.addTabbedDockWidget(dock)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         """
         Dataset setter. Saves the dataset and updates the stack with the dataset
         data
 
         :param Dataset dataset: dataset
         """
-        self.dataset = dataset
+        self._parent = parent
+        self._dataset = dataset
         self._update_dataset = dataset
         self.indices = indices
         self.setStack()
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
 
+    def _updateDataset(self, widget, dataset):
+        self._dataset = dataset
+        self._update_dataset = dataset
+        self._parent._updateDataset(widget, dataset)
+
     def setStack(self, dataset=None):
         """
         Sets new data to the stack.
@@ -135,7 +141,7 @@ class ROISelectionWidget(qt.QWidget):
         :param Dataset dataset: if not None, data set to the stack will be from the given dataset.
         """
         if dataset is None:
-            dataset = self.dataset
+            dataset = self._dataset
         first_frame_shape = dataset.get_data()[0].shape
         self.setRoi(center=(first_frame_shape[1] / 2, first_frame_shape[0] / 2),
                     size=(first_frame_shape[1] / 5, first_frame_shape[0] / 5))
@@ -203,6 +209,8 @@ class ROISelectionWidget(qt.QWidget):
         self._okB.setEnabled(True)
         if self.thread.data:
             self._update_dataset = self.thread.data
+            self.thread.func = None
+            self.thread.data = None
             self._original_dataset = False
             assert self._update_dataset is not None
             self.setStack(self._update_dataset)
@@ -222,9 +230,12 @@ class ROISelectionWidget(qt.QWidget):
             self.sigComputed.emit([], [])
 
     def getDataset(self):
-        bg_dataset = (self.bg_dataset.apply_roi(size=numpy.flip(self.roi.getSize()),
-                                                center=numpy.flip(self.roi.getCenter()))
-                      if self.bg_dataset is not None else None)
+        if self.roi:
+            bg_dataset = (self.bg_dataset.apply_roi(size=numpy.flip(self.roi.getSize()),
+                                                    center=numpy.flip(self.roi.getCenter()))
+                          if self.bg_dataset is not None else None)
+        else:
+            bg_dataset = self.bg_dataset
         return self._update_dataset, self.indices, self.bg_indices, bg_dataset
 
     def getStackViewColormap(self):
@@ -258,9 +269,9 @@ class ROISelectionWidget(qt.QWidget):
         Restores stack with the dataset data.
         """
         self.roi = None
-        self._update_dataset = self.dataset
+        self._update_dataset = self._dataset
         self._original_dataset = True
-        self.setStack(self.dataset)
+        self.setStack(self._dataset)
 
     def clearStack(self):
         """
