@@ -61,8 +61,9 @@ class ZSumWidget(qt.QMainWindow):
         widget.setLayout(layout)
         self.setCentralWidget(widget)
 
-    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+    def setDataset(self, parent, dataset, indices=None, bg_indices=None, bg_dataset=None):
         # Make sum of dataset data
+        self._parent = parent
         self.dataset = dataset
         self.indices = indices
         if len(self.dataset.data.shape) > 3:
@@ -73,8 +74,8 @@ class ZSumWidget(qt.QMainWindow):
             transformation = self.dataset.transformation.transformation
             px = transformation[0][0][0]
             py = transformation[1][0][0]
-            xscale = (transformation[0][-1][-1] - px) / len(transformation[0][0])
-            yscale = (transformation[1][-1][-1] - py) / len(transformation[1][0])
+            xscale = (transformation[0][-1][-1] - px) / transformation[0].shape[1]
+            yscale = (transformation[1][-1][-1] - py) / transformation[1].shape[0]
             self.origin = (px, py)
             self.scale = (xscale, yscale)
             label = self.dataset.transformation.label
