@@ -82,9 +82,9 @@ def createRandomDataset(dims, nb_data_files=20, header=False, _dir=None, in_memo
         # Create headers
         header = []
         # Dimensions for reshaping
-        a = numpy.random.rand(2)
-        b = numpy.random.rand(5)
-        c = numpy.random.rand(2)
+        a = sorted(numpy.random.rand(2))
+        b = [numpy.random.rand()] * numpy.array([1, 1.2, 1.4, 1.6, 1.8])
+        c = sorted(numpy.random.rand(2))
         motors = numpy.random.rand(7)
         for i in numpy.arange(nb_data_files):
             header.append({})
