@@ -33,6 +33,7 @@ from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input, Output
 from silx.gui.colors import Colormap
 from darfix.gui.blindSourceSeparationWidget import BSSWidget
+from darfix.core.process import BlindSourceSeparation
 
 
 class BlindSourceSeparationWidgetOW(OWWidget):
@@ -43,6 +44,7 @@ class BlindSourceSeparationWidgetOW(OWWidget):
     name = "blind source separation"
     icon = "icons/bss.png"
     want_main_area = False
+    ewokstaskclass = BlindSourceSeparation
 
     # Settings
     method = Setting(str(), schema_only=True)
@@ -70,13 +72,21 @@ class BlindSourceSeparationWidgetOW(OWWidget):
         self.n_comp = n_comp
 
     @Inputs.dataset
-    def setDataset(self, dataset):
-        if dataset:
+    def setDataset(self, _input):
+        try:
+            dataset, update = _input
             self._widget.setDataset(*dataset)
-            self.open()
-        self.Outputs.dataset.send(dataset)
-        self.Outputs.colormap.send(self._widget.getDisplayComponentsWidget().getStackViewColormap())
+
+            if update is None:
+                self.open()
+            self.Outputs.dataset.send(((self,) + dataset[1:], update))
+            self.Outputs.colormap.send(self._widget.getDisplayComponentsWidget().getStackViewColormap())
+        except Exception:
+            self.Outputs.dataset.send(None)
 
     @Inputs.colormap
     def setColormap(self, colormap):
         self._widget.getDisplayComponentsWidget().setStackViewColormap(colormap)
+
+    def _updateDataset(self, widget, dataset):
+        self._widget._updateDataset(widget, dataset)
