@@ -26,12 +26,11 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "03/12/2019"
+__date__ = "08/04/2020"
 
 
 from Orange.widgets.widget import OWWidget, Input, Output
 from darfix.gui.PCAWidget import PCAWidget
-from darfix.core.dataset import Dataset
 
 
 class PCAWidgetOW(OWWidget):
@@ -45,11 +44,11 @@ class PCAWidgetOW(OWWidget):
 
     # Inputs
     class Inputs:
-        dataset = Input("dataset", Dataset)
+        dataset = Input("dataset", tuple)
 
     # Outputs
     class Outputs:
-        dataset = Output("dataset", Dataset)
+        dataset = Output("dataset", tuple)
 
     def __init__(self):
         super().__init__()
@@ -58,8 +57,15 @@ class PCAWidgetOW(OWWidget):
         self.controlArea.layout().addWidget(self._widget)
 
     @Inputs.dataset
-    def setDataset(self, dataset):
-        if dataset:
-            self._widget.setDataset(dataset=dataset)
-        self.Outputs.dataset.send(dataset)
-        self.open()
+    def setDataset(self, _input):
+        try:
+            dataset, update = _input
+            self._widget.setDataset(*dataset)
+            if update is None:
+                self.open()
+            self.Outputs.dataset.send(((self,) + dataset[1:], update))
+        except Exception:
+            pass
+
+    def _updateDataset(self, widget, dataset):
+        self._widget._updateDataset(widget, dataset)
