@@ -34,6 +34,7 @@ from Orange.widgets.widget import OWWidget, Input, Output
 from silx.gui import qt
 from silx.gui.colors import Colormap
 from darfix.gui.shiftCorrectionWidget import ShiftCorrectionDialog
+from darfix.core.process import ShiftCorrection
 
 
 class ShiftCorrectionWidgetOW(OWWidget):
@@ -44,6 +45,7 @@ class ShiftCorrectionWidgetOW(OWWidget):
     name = "shift correction"
     icon = "icons/shift_correction.svg"
     want_main_area = False
+    ewokstaskclass = ShiftCorrection
 
     # Inputs
     class Inputs:
@@ -67,26 +69,34 @@ class ShiftCorrectionWidgetOW(OWWidget):
         self.controlArea.layout().addWidget(self._widget)
 
     @Inputs.dataset
-    def set_dataset(self, dataset):
-        if dataset:
+    def setDataset(self, _input):
+        try:
+            dataset, update = _input
             self._widget.setDataset(*dataset)
             if len(self.shift):
                 self._widget.mainWindow.shift = self.shift
-            self.open()
-        else:
+
+            if update is None:
+                self.open()
+            elif update != self:
+                self.Outputs.dataset.send(((self,) + self._widget.mainWindow.getDataset(), update))
+        except Exception:
             self._widget.mainWindow.clearStack()
             # Emit None
-            self.Outputs.dataset.send(dataset)
+            self.Outputs.dataset.send(None)
 
     @Inputs.colormap
     def setColormap(self, colormap):
         self._widget.setStackViewColormap(colormap)
 
+    def _updateDataset(self, widget, dataset):
+        self._widget.mainWindow._updateDataset(widget, dataset)
+
     def _sendSignal(self):
         """
         Function to emit the new dataset.
         """
         self.shift = self._widget.mainWindow.shift.tolist()
-        self.Outputs.dataset.send(self._widget.mainWindow.getDataset())
+        self.Outputs.dataset.send(((self,) + self._widget.mainWindow.getDataset(), None))
         self.Outputs.colormap.send(self._widget.mainWindow.getStackViewColormap())
         self.close()
