@@ -32,6 +32,7 @@ __date__ = "20/07/2021"
 from silx.gui.colors import Colormap
 from Orange.widgets.widget import OWWidget, Input
 from darfix.gui.zSumWidget import ZSumWidget
+from darfix.core.process import ZSum
 
 
 class ZSumWidgetOW(OWWidget):
@@ -42,6 +43,7 @@ class ZSumWidgetOW(OWWidget):
     name = "z sum"
     icon = "icons/zsum.svg"
     want_main_area = False
+    ewokstaskclass = ZSum
 
     # Inputs
     class Inputs:
@@ -55,12 +57,15 @@ class ZSumWidgetOW(OWWidget):
         self.controlArea.layout().addWidget(self._widget)
 
     @Inputs.dataset
-    def setDataset(self, dataset):
-        if dataset:
+    def setDataset(self, _input):
+        try:
+            dataset, update = _input
             self._widget.setDataset(*dataset)
-        else:
+            if update is None:
+                self.open()
+
+        except Exception:
             self._widget._plot.clear()
-        self.open()
 
     @Inputs.colormap
     def setColormap(self, colormap):
